==> autonomize_sdk_telemetry_beta-0.0.12-py3-none-any.whl: telemetry/services/logging_service.py <==
import asyncio
import traceback
from typing import Dict, Any, Optional
from opentelemetry import trace
from opentelemetry.sdk._logs import LoggerProvider, LogRecord
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry._logs.severity import SeverityNumber
from opentelemetry.trace import INVALID_SPAN_ID, INVALID_TRACE_ID, TraceFlags

from ..types import TelemetryConfig, TelemetryService
from ..constants import DEFAULT_CONFIG, ENDPOINTS
from ..utils.logging_util import custom_logger, custom_error_logger

class LoggingService(TelemetryService):
    def __init__(self, config: TelemetryConfig, resource: Resource):
        self.config = config
        self.resource = resource
        log_exporter = OTLPLogExporter(
            endpoint=f"{config.otlp_endpoint or DEFAULT_CONFIG['otlp_endpoint']}{ENDPOINTS['LOGS']}"
        )

        self.logger_provider = LoggerProvider(resource=resource)
        self.logger_provider.add_log_record_processor(
            BatchLogRecordProcessor(log_exporter)
        )
        self.logger = self.logger_provider.get_logger(config.service_name)

    def _emit_log(
        self,
        severity: SeverityNumber,
        message: str,
        attributes: Optional[Dict[str, Any]] = None
    ) -> None:
        if attributes is None:
            attributes = {}

        span = trace.get_current_span()
        context = span.get_span_context() if span else None
        
        # Get trace context data with defaults
        trace_id = context.trace_id if context and context.is_valid else INVALID_TRACE_ID
        span_id = context.span_id if context and context.is_valid else INVALID_SPAN_ID
        trace_flags = context.trace_flags if context and context.is_valid else TraceFlags(0)

        try:
            self.logger.emit(
                LogRecord(
                    timestamp=None,
                    severity_number=severity,
                    severity_text=severity.name,
                    body=message,
                    attributes=attributes,
                    trace_id=trace_id,
                    span_id=span_id,
                    trace_flags=trace_flags,
                    resource=self.resource  # Add the resource to the LogRecord
                )
            )
        except Exception as error:
            custom_error_logger(f"Failed to emit log: {error}")

    async def start(self) -> None:
        try:
            await self._test()
            custom_logger("Logging service started successfully")
        except Exception as error:
            custom_error_logger("Failed to start logging service:", error)
            raise

    async def _test(self) -> None:
        try:
            self.info("Telemetry logging service initialized", {
                "status": "initialized"
            })
            await asyncio.sleep(0.1)  # Allow time for log to be processed
            custom_logger("Test log sent successfully")
        except Exception as error:
            custom_error_logger("Failed to send test log:", error)
            raise

    async def shutdown(self) -> None:
        try:
            await self.logger_provider.shutdown()
        except Exception as error:
            custom_error_logger("Failed to shutdown logging service:", error)
            raise

    def info(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.INFO, message, attributes)

    def error(
        self,
        message: str,
        error: Optional[Exception] = None,
        attributes: Optional[Dict[str, Any]] = None
    ) -> None:
        if attributes is None:
            attributes = {}

        if error:
            attributes.update({
                "error.type": error.__class__.__name__,
                "error.message": str(error),
                "error.stack_trace": traceback.format_exc()
            })

        self._emit_log(SeverityNumber.ERROR, message, attributes)

    def warn(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.WARN, message, attributes)

    def debug(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.DEBUG, message, attributes)

==> autonomize_sdk_telemetry_beta-0.0.14-py3-none-any.whl: telemetry/services/logging_service.py <==
import asyncio
from datetime import datetime
import traceback
from typing import Dict, Any, Optional
from opentelemetry import trace
from opentelemetry.sdk._logs import LoggerProvider, LogRecord
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry._logs.severity import SeverityNumber
from opentelemetry.trace import INVALID_SPAN_ID, INVALID_TRACE_ID, TraceFlags

from ..types import TelemetryConfig, TelemetryService
from ..constants import DEFAULT_CONFIG, ENDPOINTS
from ..utils.logging_util import custom_logger, custom_error_logger

class LoggingService(TelemetryService):
    def __init__(self, config: TelemetryConfig, resource: Resource):
        self.config = config
        self.resource = resource
        log_exporter = OTLPLogExporter(
            endpoint=f"{config.otlp_endpoint or DEFAULT_CONFIG['otlp_endpoint']}{ENDPOINTS['LOGS']}"
        )

        self.logger_provider = LoggerProvider(resource=resource)
        self.logger_provider.add_log_record_processor(
            BatchLogRecordProcessor(log_exporter)
        )
        self.logger = self.logger_provider.get_logger(config.service_name)

    def _emit_log(
        self,
        severity: SeverityNumber,
        message: str,
        attributes: Optional[Dict[str, Any]] = None
    ) -> None:
        if attributes is None:
            attributes = {}

        span = trace.get_current_span()
        context = span.get_span_context() if span else None
        
        # Get trace context data with defaults
        trace_id = context.trace_id if context and context.is_valid else INVALID_TRACE_ID
        span_id = context.span_id if context and context.is_valid else INVALID_SPAN_ID
        trace_flags = context.trace_flags if context and context.is_valid else TraceFlags(0)

        try:
            self.logger.emit(
                LogRecord(
                    timestamp=datetime.now(),
                    severity_number=severity,
                    severity_text=severity.name,
                    body=message,
                    attributes=attributes,
                    trace_id=trace_id,
                    span_id=span_id,
                    trace_flags=trace_flags
                )
            )
        except Exception as error:
            custom_error_logger(f"Failed to emit log: {error}")

    async def start(self) -> None:
        try:
            await self._test()
            custom_logger("Logging service started successfully")
        except Exception as error:
            custom_error_logger("Failed to start logging service:", error)
            raise

    async def _test(self) -> None:
        try:
            self.info("Telemetry logging service initialized", {
                "status": "initialized"
            })
            await asyncio.sleep(0.1)  # Allow time for log to be processed
            custom_logger("Test log sent successfully")
        except Exception as error:
            custom_error_logger("Failed to send test log:", error)
            raise

    async def shutdown(self) -> None:
        try:
            await self.logger_provider.shutdown()
        except Exception as error:
            custom_error_logger("Failed to shutdown logging service:", error)
            raise

    def info(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.INFO, message, attributes)

    def error(
        self,
        message: str,
        error: Optional[Exception] = None,
        attributes: Optional[Dict[str, Any]] = None
    ) -> None:
        if attributes is None:
            attributes = {}

        if error:
            attributes.update({
                "error.type": error.__class__.__name__,
                "error.message": str(error),
                "error.stack_trace": traceback.format_exc()
            })

        self._emit_log(SeverityNumber.ERROR, message, attributes)

    def warn(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.WARN, message, attributes)

    def debug(self, message: str, attributes: Optional[Dict[str, Any]] = None) -> None:
        self._emit_log(SeverityNumber.DEBUG, message, attributes)
